-- FUNCTION: laboreo_pwa.f_empleado_validation(character varying, character varying)

-- DROP FUNCTION laboreo_pwa.f_empleado_validation(character varying, character varying);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_validation(
	pidentificador character varying,
	ppin character varying)
    RETURNS TABLE(idempleado bigint, dni character varying, nombre character varying, foto text, administrador boolean, supervisor boolean, iddepartamento bigint, idorganizacion bigint) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN
  RETURN QUERY
	-- Validamos la existencia del usuario por el identificador (dni) y el pin
	SELECT empleado.idEmpleado, empleado.dni, empleado.nombre, empleado.foto, rolUsuario.administrador, rolUsuario.supervisor, empleado.iddepartamento, empleado.idorganizacion
	FROM laboreo_pwa.empleado
	INNER JOIN laboreo_pwa.rolUsuario
		ON rolUsuario.idRolUsuario = empleado.idRolUsuario
	WHERE empleado.dni = pidentificador
		AND empleado.pin = ppin
	UNION 
	SELECT 0, '00000000X', 'Administrador', NULL, TRUE, FALSE, 0, null		
	WHERE UPPER(pidentificador) = 'ADMIN'
		AND UPPER(ppin) = '2021';
		
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado_validation(character varying, character varying)
    OWNER TO prbnnoeimelclv;





/** Eventos del calendario **/
-- FUNCTION: laboreo_pwa.f_empleado_calendarioevento(bigint, timestamp without time zone, timestamp without time zone)
-- DROP FUNCTION laboreo_pwa.f_empleado_calendarioevento(bigint, timestamp without time zone, timestamp without time zone);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_calendarioevento(
	pidempleado bigint,
	pfechainicio timestamp without time zone,
	pfechafin timestamp without time zone)
    RETURNS TABLE(idempleado bigint, idempleadoacceso bigint, empleado character varying, imagen text, idevento bigint, fechainicio timestamp without time zone, fechafin timestamp without time zone, eventonombre character varying, eventotipo character varying) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN
  RETURN QUERY
  
	/*WITH days AS (
		select * 
		from generate_series(pfechainicio, pfechafin, '1d'::interval) d
	)
	
	SELECT	pidempleado AS idempleado,
			days.d::TIMESTAMP WITHOUT TIME ZONE AS fechainicio,
			days.d::TIMESTAMP WITHOUT TIME ZONE AS fechafin,
			null AS eventonombre,
			'calendario' AS eventotipo
	FROM days
	UNION*/
	
	SELECT festivo.idempleado,
		empleadov.idempleadoacceso,
		empleadov.nombre AS empleado,
		empleadov.imagen,
		festivo.idevento,
		festivo.fechainicio,
		festivo.fechafin,
		'Vacaciones'::varchar AS eventonombre,
		'festivo'::varchar AS eventotipo
	FROM laboreo_pwa.festivo 
	INNER JOIN laboreo_pwa.f_empleado(pidempleado) empleadov
		ON empleadov.idempleado = festivo.idempleado
	WHERE (pfechainicio BETWEEN festivo.fechainicio::date AND festivo.fechafin::date)
		AND (pfechafin BETWEEN festivo.fechainicio::date AND festivo.fechafin::date)
	UNION
	SELECT registrohorario.idempleado,
		empleadov.idempleadoacceso,
		empleadov.nombre AS empleado,
		empleadov.imagen,
		registrohorario.idevento,
		registrohorario.fechainicio,
		registrohorario.fechafin,
		CASE
			WHEN registrohorario.entrada THEN 'Entrada'::varchar
			ELSE 'Salida'::varchar
		END AS eventonombre,
		'registrohorario'::varchar AS eventotipo
	FROM laboreo_pwa.registrohorario 
	INNER JOIN laboreo_pwa.f_empleado(pidempleado) empleadov
		ON empleadov.idempleado = registrohorario.idempleado
	WHERE (pfechainicio BETWEEN registrohorario.fechainicio::date AND registrohorario.fechafin::date)
		AND (pfechafin BETWEEN registrohorario.fechainicio::date AND registrohorario.fechafin::date)
	UNION
	SELECT ausencia.idempleado,
		empleadov.idempleadoacceso,
		empleadov.nombre AS empleado,
		empleadov.imagen,	
		ausencia.idevento,
		ausencia.fechainicio,
		ausencia.fechafin,
		ausencia.motivo AS eventonombre,
		'ausencia'::varchar AS eventotipo
	FROM laboreo_pwa.ausencia
	INNER JOIN laboreo_pwa.f_empleado(pidempleado) empleadov
		ON empleadov.idempleado = ausencia.idempleado
	WHERE (pfechainicio BETWEEN ausencia.fechainicio::date AND ausencia.fechafin::date)
		AND (pfechafin BETWEEN ausencia.fechainicio::date AND ausencia.fechafin::date)
	UNION
	SELECT personal.idempleado,
		empleadov.idempleadoacceso,
		empleadov.nombre AS empleado,
		empleadov.imagen,
		personal.idevento,
		personal.fechainicio,
		personal.fechafin,
		personal.titulo AS eventonombre,
		'personal'::varchar AS eventotipo
	FROM laboreo_pwa.personal 
	INNER JOIN laboreo_pwa.empleadov
		ON empleadov.idempleado = personal.idempleado
	WHERE (pfechainicio BETWEEN personal.fechainicio::date AND personal.fechafin::date)
		AND (pfechafin BETWEEN personal.fechainicio::date AND personal.fechafin::date)
		AND empleadov.idempleado = pidempleado
	UNION
	SELECT jornada.idempleado,
		empleadov.idempleadoacceso,
		empleadov.nombre AS empleado,
		empleadov.imagen,
		null,
		dias.d,
		dias.d,
		turno.nombre AS eventonombre,
		'jornada'::varchar AS eventotipo	
	FROM laboreo_pwa.jornada
	INNER JOIN laboreo_pwa.empleadov
		ON empleadov.idempleado = jornada.idempleado
	INNER JOIN laboreo_pwa.turno
		ON turno.idturno = jornada.idturno
	INNER JOIN generate_series(pfechainicio, pfechafin, '1d'::interval) AS dias(d)
		ON 
			(EXTRACT(dow FROM dias.d) = 1 AND turno.lunes = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 2 AND turno.martes = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 3 AND turno.miercoles = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 4 AND turno.jueves = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 5 AND turno.viernes = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 6 AND turno.sabado = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 7 AND turno.domingo = TRUE)
	WHERE jornada.idempleado = pidempleado
		AND dias.d::DATE BETWEEN pfechainicio::DATE AND pfechafin::DATE;		
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado_calendarioevento(bigint, timestamp without time zone, timestamp without time zone)
    OWNER TO prbnnoeimelclv;
/** Calendario Eventos **/




/** Calendario **/
-- FUNCTION: laboreo_pwa.f_empleado_calendario(bigint, timestamp without time zone, timestamp without time zone)

-- DROP FUNCTION laboreo_pwa.f_empleado_calendario(bigint, timestamp without time zone, timestamp without time zone);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_calendario(
	pidempleado bigint,
	pfechainicio timestamp without time zone,
	pfechafin timestamp without time zone)
    RETURNS TABLE(idempleado bigint, dia integer, fecha character varying, registrohorario boolean, festivo boolean, ausencia boolean, personal boolean, jornada boolean) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN  
	CREATE TEMPORARY TABLE temp_dias (
    	idempleado bigint, 
		dia INTEGER, 
		fecha DATE,
	  	registrohorario boolean, 
	  	festivo boolean, 
	  	ausencia boolean, 
	  	personal boolean,
		jornada boolean
   	);
  
	WITH days AS (
		select * 
		from generate_series(pfechainicio, pfechafin, '1d'::interval) d
	)
	
	INSERT INTO temp_dias
		SELECT	pidempleado AS idempleado,
				date_part('day', days.d)::INTEGER AS dia,
				days.d::DATE,
				false,
				false,
				false,
				false,
				false
		FROM days;
		
	UPDATE temp_dias
	SET festivo = true
	FROM laboreo_pwa.festivo 
	INNER JOIN laboreo_pwa.f_empleado(pidempleado) empleadov
		ON empleadov.idempleado = festivo.idempleado
	WHERE temp_dias.fecha BETWEEN festivo.fechainicio::DATE AND festivo.fechafin::DATE
		AND (festivo.autorizada = TRUE OR (festivo.autorizada = FALSE AND festivo.fechaautorizacion IS NULL));
	
	UPDATE temp_dias
	SET registrohorario = true
	FROM laboreo_pwa.registrohorario 
	INNER JOIN laboreo_pwa.f_empleado(pidempleado) empleadov
		ON empleadov.idempleado = registrohorario.idempleado			
	WHERE temp_dias.fecha BETWEEN registrohorario.fechainicio::DATE AND registrohorario.fechafin::DATE;
	
	UPDATE temp_dias
	SET ausencia = true
	FROM laboreo_pwa.ausencia 
	INNER JOIN laboreo_pwa.f_empleado(pidempleado) empleadov
		ON empleadov.idempleado = ausencia.idempleado			
	WHERE temp_dias.fecha BETWEEN ausencia.fechainicio::DATE AND ausencia.fechafin::DATE;

	UPDATE temp_dias
	SET personal = true
	FROM laboreo_pwa.personal 
	INNER JOIN laboreo_pwa.empleadov
		ON empleadov.idempleado = personal.idempleado
	WHERE temp_dias.fecha BETWEEN personal.fechainicio::DATE AND personal.fechafin::DATE
		AND empleadov.idempleado = pidempleado;
			

	UPDATE temp_dias
	SET jornada = true
	FROM laboreo_pwa.jornada
	INNER JOIN laboreo_pwa.turno
		ON turno.idturno = jornada.idturno
	INNER JOIN generate_series(jornada.fechainicio, jornada.fechafin, '1d'::interval) AS dias(d)
		ON 	(EXTRACT(dow FROM dias.d) = 1 AND turno.lunes = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 2 AND turno.martes = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 3 AND turno.miercoles = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 4 AND turno.jueves = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 5 AND turno.viernes = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 6 AND turno.sabado = TRUE) OR
			(EXTRACT(dow FROM dias.d) = 7 AND turno.domingo = TRUE)
	WHERE jornada.idempleado = pidempleado
		AND temp_dias.fecha BETWEEN jornada.fechainicio::DATE AND jornada.fechafin::DATE
		AND dias.d::DATE = temp_dias.fecha::DATE;
  RETURN QUERY

	SELECT temp_dias.idempleado, temp_dias.dia, TO_CHAR(temp_dias.fecha, 'YYYYMMDD')::varchar(8), temp_dias.registrohorario, temp_dias.festivo, temp_dias.ausencia, temp_dias.personal, temp_dias.jornada
	FROM temp_dias;	
	
	DROP TABLE temp_dias;
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado_calendario(bigint, timestamp without time zone, timestamp without time zone)
    OWNER TO prbnnoeimelclv;

/** Calendario **/



/* Función para recuperar eventos dado el id */
-- FUNCTION: laboreo_pwa.f_evento(bigint)

-- DROP FUNCTION laboreo_pwa.f_evento(bigint);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_evento(
	pidevento bigint)
    RETURNS TABLE(idevento bigint, fechainicio character varying, fechafin character varying, entrada boolean, longitud double precision, latitud double precision, autorizada boolean, fechaautorizacion character varying, motivorechazo character varying, motivo character varying, minutos integer, titulo character varying, descripcion character varying, idempleado bigint, idempleado_autoriza bigint, tipoevento character varying) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN  
	CREATE TEMPORARY TABLE temp_evento (
    	idevento bigint, 
		fechainicio TIMESTAMP WITHOUT TIME ZONE, 
		fechafin TIMESTAMP WITHOUT TIME ZONE, 
		entrada BOOLEAN, 
		longitud FLOAT, 
		latitud FLOAT, 
		autorizada BOOLEAN, 
		fechaautorizacion TIMESTAMP WITHOUT TIME ZONE, 
		motivorechazo VARCHAR, 
		motivo VARCHAR, 
		minutos INT, 
		titulo VARCHAR, 
		descripcion VARCHAR, 
		idempleado BIGINT, 
		idempleado_autoriza BIGINT,
		tipoevento VARCHAR
   	);
  
	
	INSERT INTO temp_evento (idevento, fechainicio, fechafin, idempleado)
	SELECT	evento.idevento,
			evento.fechainicio,
			evento.fechafin,
			evento.idempleado
	FROM laboreo_pwa.evento
	WHERE evento.idevento = pidevento;
		
	UPDATE temp_evento
	SET autorizada = festivo.autorizada, fechaautorizacion = festivo.fechaautorizacion, motivorechazo = festivo.motivorechazo, tipoevento = 'festivo', idempleado_autoriza = festivo.idempleado_autoriza
	FROM laboreo_pwa.festivo 
	WHERE festivo.idevento = temp_evento.idevento;
	
	UPDATE temp_evento
	SET entrada = registrohorario.entrada, longitud = registrohorario.longitud, latitud = registrohorario.latitud, tipoevento = 'registrohorario'
	FROM laboreo_pwa.registrohorario 
	WHERE registrohorario.idevento = temp_evento.idevento;
			
	UPDATE temp_evento
	SET motivo = ausencia.motivo, minutos = ausencia.minutos, tipoevento = 'ausencia'
	FROM laboreo_pwa.ausencia 
	WHERE ausencia.idevento = temp_evento.idevento;

	UPDATE temp_evento
	SET titulo = personal.titulo, descripcion = personal.descripcion, tipoevento = 'personal'
	FROM laboreo_pwa.personal 
	WHERE personal.idevento = temp_evento.idevento;

  RETURN QUERY

	SELECT temp_evento.idevento, 		
		to_char(temp_evento.fechainicio, 'YYYY-MM-DD"T"HH24:MI:SS')::varchar AS fechainicio,
		to_char(temp_evento.fechafin, 'YYYY-MM-DD"T"HH24:MI:SS')::varchar AS fechafin, 
		temp_evento.entrada, 
		temp_evento.longitud, 
		temp_evento.latitud, 
		temp_evento.autorizada, 
		to_char(temp_evento.fechaautorizacion, 'YYYY-MM-DD')::varchar AS fechaautorizacion, 
		temp_evento.motivorechazo, 
		temp_evento.motivo, 
		temp_evento.minutos, 
		temp_evento.titulo, 
		temp_evento.descripcion, 
		temp_evento.idempleado, 
		temp_evento.idempleado_autoriza, 
		temp_evento.tipoevento
	FROM temp_evento;	
	
	DROP TABLE temp_evento;
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_evento(bigint)
    OWNER TO prbnnoeimelclv;

/* Función para recuperar eventos dado el id */




-- Función para devolver los contratos de un empleado
CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_contratos(
	pidempleado bigint)
    RETURNS TABLE(idcontratoempleado bigint, fechainicio character varying, fechafin character varying, nombre character varying, codigo character varying) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN  

  RETURN QUERY

	SELECT contratoempleado.idcontratoempleado,
		to_char(contratoempleado.fechainicio, 'YYYY-MM-DD"T"HH24:MI:SS')::varchar AS fechainicio,
		to_char(contratoempleado.fechafin, 'YYYY-MM-DD"T"HH24:MI:SS')::varchar AS fechafin, 
		tipocontrato.nombre, 
		tipocontrato.codigo
	FROM laboreo_pwa.contratoempleado
	INNER JOIN laboreo_pwa.tipocontrato ON tipocontrao.idtipocontrato = contratoempleado.idtipocontrato
	WHERE contratoempleado.idempleado = pidempleado;	
		
END; $BODY$;


-- Función para devolver las jornadas de un empleado
CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_jornadas(
	pidempleado bigint)
    RETURNS TABLE(idjornada bigint, fechainicio character varying, fechafin character varying, nombre character varying, codigo character varying) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN  

  RETURN QUERY

	SELECT jornada.idjornada,
		to_char(jornada.fechainicio, 'YYYY-MM-DD')::varchar AS fechainicio,
		to_char(jornada.fechafin, 'YYYY-MM-DD')::varchar AS fechafin, 
		turno.nombre, 
		turno.codigo
	FROM laboreo_pwa.jornada
	INNER JOIN laboreo_pwa.turno ON turno.idturno = jornada.idturno
	WHERE jornada.idempleado = pidempleado;	
		
END; $BODY$;






/** Listado Resumen Mes **/
-- FUNCTION: laboreo_pwa.f_empleado_resumenmes(bigint, timestamp without time zone, timestamp without time zone)
-- DROP FUNCTION laboreo_pwa.f_empleado_resumenmes(bigint, timestamp without time zone, timestamp without time zone);
CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_resumenmes(
	pidempleado bigint,
	pfechainicio timestamp without time zone,
	pfechafin timestamp without time zone)
    RETURNS TABLE(idempleado bigint, dia integer, fecha date, inicio character varying, fin character varying, pausas character varying, correcto boolean, trabajadas character varying) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
DECLARE 
	reg        	RECORD;
	reg2		RECORD;
	fentrada	TIMESTAMP WITHOUT TIME ZONE;
	fsalida		TIMESTAMP WITHOUT TIME ZONE;
	fsegundos		FLOAT;
	fcorrecto		BOOLEAN;
	ventrada		VARCHAR(10);
	vsalida		VARCHAR(10);
	vpausas		VARCHAR(80);
	cursordias CURSOR FOR SELECT temp_dias.fecha FROM temp_dias;
BEGIN
	DROP TABLE IF EXISTS temp_dias;
	CREATE TEMPORARY TABLE temp_dias (
		idempleado bigint, 
		dia INTEGER, 
		fecha DATE,
		correcto BOOLEAN,
		tiempojornada varchar(8),
		inicio VARCHAR(10),
		fin VARCHAR(10),
		pausas VARCHAR(80)
	);
	DROP TABLE IF EXISTS temp_entradas;
	CREATE TEMPORARY TABLE temp_entradas (
		idempleado bigint, 
		dia INTEGER, 
		fecha TIMESTAMP
	);
	DROP TABLE IF EXISTS temp_salidas;
	CREATE TEMPORARY TABLE temp_salidas (
		idempleado bigint, 
		dia INTEGER, 
		fecha TIMESTAMP
	);

	WITH days AS (
		select * 
		from generate_series('20210501', '20210531', '1d'::interval) d
	)

	INSERT INTO temp_dias (idempleado, dia, fecha)
	SELECT	pidempleado AS idempleado,
			date_part('day', days.d)::INTEGER AS dia,
			days.d::DATE
	FROM days;

	INSERT INTO temp_entradas
	SELECT registrohorario.idempleado, temp_dias.dia, min(registrohorario.fechainicio)
	FROM laboreo_pwa.registrohorario 
	INNER JOIN temp_dias ON registrohorario.idempleado = temp_dias.idempleado 
		AND temp_dias.fecha::DATE BETWEEN registrohorario.fechainicio::DATE AND registrohorario.fechafin::DATE		
		AND registrohorario.entrada = true		
	GROUP BY registrohorario.idempleado, temp_dias.dia;	

	INSERT INTO temp_salidas
	SELECT registrohorario.idempleado, temp_dias.dia, max(registrohorario.fechainicio)
	FROM laboreo_pwa.registrohorario 
	INNER JOIN temp_dias ON registrohorario.idempleado = temp_dias.idempleado 
		AND temp_dias.fecha BETWEEN registrohorario.fechainicio::DATE AND registrohorario.fechafin::DATE
		AND registrohorario.entrada = false		
	GROUP BY registrohorario.idempleado, temp_dias.dia;

	OPEN cursordias;
   	FETCH cursordias INTO reg;
   	WHILE( FOUND ) LOOP
       	RAISE NOTICE ' PROCESANDO  %', reg.fecha;
		
		fsegundos = 0;
		fentrada = null;
		fsalida = null;
		ventrada = null;
		vsalida = null;
		vpausas = '';
		
 		FOR reg2 IN SELECT fechainicio, entrada FROM laboreo_pwa.registrohorario WHERE registrohorario.idempleado = pidempleado AND fechainicio::DATE = reg.fecha::DATE ORDER BY fechainicio ASC  LOOP 
			IF reg2.entrada = true THEN
				fentrada = reg2.fechainicio;
				IF ventrada IS NULL THEN 
					ventrada = to_char(reg2.fechainicio, 'HH24:MI');
				END IF;
			ELSE
				fsalida = reg2.fechainicio;
				vsalida = to_char(reg2.fechainicio, 'HH24:MI');
			END IF;		
			vpausas = vpausas || to_char(reg2.fechainicio, 'HH24:MI') || ' / ';
			
			
			IF (fentrada IS NOT NULL AND fsalida IS NOT NULL) THEN			
				--RAISE NOTICE 'TIME %', extract('epoch' from fsalida - fentrada) * interval '1 sec';			
				fsegundos = fsegundos + extract('epoch' from fsalida - fentrada);
				fentrada = NULL;
				fsalida = NULL;								
			END IF;
       		
   		END LOOP;
		fcorrecto = TRUE;
		IF (fentrada IS NOT NULL AND fsalida IS NULL) or (fentrada IS NULL AND fsalida IS NOT NULL) THEN
			fsegundos = 0;
			fcorrecto = FALSE;
			vsalida = '';
		END IF;
		
		RAISE NOTICE 'SEC %', (fsegundos * interval '1 second');				
		UPDATE temp_dias
		SET tiempojornada = to_char(fsegundos * interval '1 sec', 'HH24:MI'), correcto = fcorrecto,
				inicio = ventrada, fin = vsalida, pausas = vpausas
		WHERE temp_dias.idempleado = pidempleado 
			AND temp_dias.fecha::DATE = reg.fecha::DATE;

       	FETCH cursordias INTO reg;
   	END LOOP ;
   	RETURN QUERY
	SELECT 	temp_dias.idempleado, 
			temp_dias.dia, 
			temp_dias.fecha, 
			temp_dias.inicio,
			temp_dias.fin,
			temp_dias.pausas,
			temp_dias.correcto, 			
			temp_dias.tiempojornada	
	FROM temp_dias;
END;
$BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado_resumenmes(bigint, timestamp without time zone, timestamp without time zone)
    OWNER TO prbnnoeimelclv;
/** Listado Resumen Mes **/



/** Turnos de los empleados **/
CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_turnos(
	pidempleado bigint)
    RETURNS TABLE(idempleado bigint, idevento bigint, fechainicio TIMESTAMP WITHOUT TIME ZONE, fechafin TIMESTAMP WITHOUT TIME ZONE, eventonombre character varying, eventotipo character varying) 
    LANGUAGE 'plpgsql'
   
AS $BODY$
BEGIN
  RETURN QUERY
  	
	SELECT 	jornada.idempleado,
			(days.d + turno.horainicio)::TIMESTAMP WITHOUT TIME ZONE, 
			(days.d + turno.horafin)::TIMESTAMP WITHOUT TIME ZONE
	FROM laboreo_pwa.jornada
	INNER JOIN laboreo_pwa.turno
		ON turno.idturno = jornada.idturno
	INNER JOIN generate_series(jornada.fechainicio, jornada.fechafin, '1d'::interval) AS days(d)
		ON (EXTRACT(dow FROM days.d)+1 = 1 AND turno.lunes = TRUE) OR
			(EXTRACT(dow FROM days.d)+1 = 2 AND turno.martes = TRUE) OR
			(EXTRACT(dow FROM days.d)+1 = 3 AND turno.miercoles = TRUE) OR
			(EXTRACT(dow FROM days.d)+1 = 4 AND turno.jueves = TRUE) OR
			(EXTRACT(dow FROM days.d)+1 = 5 AND turno.viernes = TRUE) OR
			(EXTRACT(dow FROM days.d)+1 = 6 AND turno.sabado = TRUE) OR
			(EXTRACT(dow FROM days.d)+1 = 7 AND turno.domingo = TRUE)
	WHERE jornada.idempleado = pidempleado;

END; $BODY$;


/** Consulta de empleados segun solicitante **/
-- FUNCTION: laboreo_pwa.f_empleado(bigint)
-- DROP FUNCTION laboreo_pwa.f_empleado(bigint);
CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado(
	pidempleado bigint)
    RETURNS TABLE(idempleado bigint, idorganizacion bigint, iddepartamento bigint, idconvenio bigint, idcontratoempleado_actual bigint, idrolusuario bigint, nombre character varying, apellidos character varying, dni character varying, telefono character varying, email character varying, observaciones text, foto text, numeroinss character varying, direccion character varying, codigopostal character varying, poblacion character varying, pais character varying, imagen text, idcentrotrabajo bigint, idempleadoacceso bigint) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
DECLARE 
	useradmin BOOLEAN;
	usersupervisor BOOLEAN;
BEGIN
	SELECT rolusuario.administrador, rolusuario.supervisor
	INTO useradmin, usersupervisor
	FROM laboreo_pwa.empleado
	INNER JOIN laboreo_pwa.rolusuario
		ON rolusuario.idrolusuario = empleado.idrolusuario
	WHERE empleado.idempleado = pidempleado;
	
  RETURN QUERY
	-- Validamos la existencia del usuario por el identificador (dni) y el pin
	SELECT empleado.idempleado,
    empleado.idorganizacion,
    empleado.iddepartamento,
    empleado.idconvenio,
    empleado.idcontratoempleado_actual,
    empleado.idrolusuario,
    empleado.nombre,
	empleado.apellidos,
    empleado.dni,
    empleado.telefono,
    empleado.email,
    empleado.observaciones,    
    empleado.foto,
    empleado.numeroinss,
    empleado.direccion,
    empleado.codigopostal,
    empleado.poblacion,
    empleado.pais,    
    COALESCE(empleado.foto, 'img/avatar-noimage.png'::character varying::text) AS imagen,
    departamento.idcentrotrabajo,
    empleadosupervisor.idempleado AS idempleadoacceso    
   FROM laboreo_pwa.empleado
     JOIN laboreo_pwa.departamento ON departamento.iddepartamento = empleado.iddepartamento
     LEFT JOIN laboreo_pwa.empleado empleadosupervisor ON empleadosupervisor.idempleado = departamento.idempleado_responsable
   WHERE empleado.idempleado = pidempleado
   		OR empleadosupervisor.idempleado = pidempleado
		OR useradmin = true;
		
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado(bigint)
    OWNER TO prbnnoeimelclv;


/** Función para obtener centros de trabajo según el empleado **/
-- FUNCTION: laboreo_pwa.f_centrotrabajo(bigint)
-- DROP FUNCTION laboreo_pwa.f_centrotrabajo(bigint);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_centrotrabajo(
	pidempleado bigint)
    RETURNS TABLE(idcentrotrabajo bigint, idorganizacion bigint, codigo character varying, nombre character varying, direccion character varying, codigopostal character varying, poblacion character varying, pais character varying, telefono character varying) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
DECLARE 
	useradmin BOOLEAN;
	usersupervisor BOOLEAN;
BEGIN
	SELECT rolusuario.administrador, rolusuario.supervisor
	INTO useradmin, usersupervisor
	FROM laboreo_pwa.empleado
	INNER JOIN laboreo_pwa.rolusuario
		ON rolusuario.idrolusuario = empleado.idrolusuario
	WHERE empleado.idempleado = pidempleado;
	
  RETURN QUERY
	-- Validamos la existencia del usuario por el identificador (dni) y el pin
	SELECT DISTINCT centrotrabajo.idcentrotrabajo,
    centrotrabajo.idorganizacion,
	centrotrabajo.codigo,
    centrotrabajo.nombre,
    centrotrabajo.direccion,
    centrotrabajo.codigopostal,
    centrotrabajo.poblacion,
    centrotrabajo.pais,    
    centrotrabajo.telefono 	
   FROM laboreo_pwa.centrotrabajo
     JOIN laboreo_pwa.departamento ON departamento.idcentrotrabajo = centrotrabajo.idcentrotrabajo
     JOIN laboreo_pwa.empleado ON empleado.iddepartamento = departamento.iddepartamento
   WHERE empleado.idempleado = pidempleado   		
		OR useradmin = true;
		
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_centrotrabajo(bigint)
    OWNER TO prbnnoeimelclv;



/** Función para obtener departamentos según el empleado **/
-- FUNCTION: laboreo_pwa.f_departamento(bigint)

-- DROP FUNCTION laboreo_pwa.f_departamento(bigint);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_departamento(
	pidempleado bigint)
    RETURNS TABLE(iddepartamento bigint, idempleado_responsable bigint, codigo character varying, nombre character varying, idcentrotrabajo bigint) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
DECLARE 
	useradmin BOOLEAN;
	usersupervisor BOOLEAN;
BEGIN
	SELECT rolusuario.administrador, rolusuario.supervisor
	INTO useradmin, usersupervisor
	FROM laboreo_pwa.empleado
	INNER JOIN laboreo_pwa.rolusuario
		ON rolusuario.idrolusuario = empleado.idrolusuario
	WHERE empleado.idempleado = pidempleado;
	
  RETURN QUERY
	-- Validamos la existencia del usuario por el identificador (dni) y el pin
	SELECT DISTINCT departamento.iddepartamento,
    departamento.idempleado_responsable,
	departamento.codigo,
    departamento.nombre,
    departamento.idcentrotrabajo
   FROM laboreo_pwa.departamento
     JOIN laboreo_pwa.empleado ON empleado.iddepartamento = departamento.iddepartamento
   WHERE empleado.idempleado = pidempleado   		
		OR useradmin = true;
		
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_departamento(bigint)
    OWNER TO prbnnoeimelclv;




/** Status del empleado **/
-- FUNCTION: laboreo_pwa.f_empleado(bigint)
-- DROP FUNCTION laboreo_pwa.f_empleado(bigint);

CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_status(
	pidempleado bigint)
    RETURNS TABLE(idempleado bigint, pendientes integer, consumidos integer, diasnaturalesvacaciones integer, diaslaborablesvacaciones integer) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN

  RETURN QUERY
	-- Validamos la existencia del usuario por el identificador (dni) y el pin
	SELECT empleado.idempleado,
		aviso.pendientes, 		
		COALESCE(vacaciones.consumidos::INTEGER, 0) AS consumidos, 
		convenio.diasnaturalesvacaciones, 
		convenio.diaslaborablesvacaciones
   FROM laboreo_pwa.empleado
    LEFT JOIN (
			SELECT COUNT(aviso.*)::INTEGER AS pendientes, aviso.idempleado
			FROM laboreo_pwa.aviso
			WHERE aviso.idempleado = pidempleado
				AND aviso.leido = false
			GROUP BY aviso.idempleado
	) aviso
		ON aviso.idempleado = empleado.idempleado			
	LEFT JOIN laboreo_pwa.convenio
		ON convenio.idconvenio = empleado.idconvenio
	LEFT JOIN (
			SELECT SUM((festivo.fechafin::DATE - festivo.fechainicio::DATE)+1) AS consumidos, festivo.idempleado
			FROM laboreo_pwa.festivo
			WHERE extract(year from festivo.fechainicio) = extract(year from current_timestamp)
				AND festivo.idempleado = pidempleado
			GROUP BY festivo.idempleado
	) vacaciones
		ON vacaciones.idempleado = empleado.idempleado	 
   WHERE empleado.idempleado = pidempleado;
		
END; $BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado_status(bigint)
    OWNER TO prbnnoeimelclv;
/** Status del empleado **/



/** Resumen vacaciones **/
-- FUNCTION: laboreo_pwa.f_empleado_resumenfestivo(bigint, timestamp without time zone)
-- DROP FUNCTION laboreo_pwa.f_empleado_resumenfestivo(bigint, timestamp without time zone);
CREATE OR REPLACE FUNCTION laboreo_pwa.f_empleado_resumenfestivo(
	pidempleado bigint,
	pfecha timestamp without time zone)
    RETURNS TABLE(idempleado bigint, nombre character varying, apellidos character varying, imagen text, diasnaturalesvacaciones integer, consumidos integer, restantes integer) 
    LANGUAGE 'plpgsql'

    COST 100
    VOLATILE 
    ROWS 1000
    
AS $BODY$
BEGIN	
   	RETURN QUERY
SELECT empleado.idempleado,
		empleado.nombre,
		empleado.apellidos,
		empleado.imagen,
		convenio.diasnaturalesvacaciones, 		
		COALESCE(vacaciones.consumidos::INTEGER, 0) AS consumidos, 
		(convenio.diasnaturalesvacaciones - vacaciones.consumidos)::INTEGER		
   	FROM laboreo_pwa.f_empleado(pidempleado) empleado
	LEFT JOIN laboreo_pwa.convenio
		ON convenio.idconvenio = empleado.idconvenio
	LEFT JOIN (
			SELECT SUM((festivo.fechafin::DATE - festivo.fechainicio::DATE)+1) AS consumidos, festivo.idempleado
			FROM laboreo_pwa.f_empleado(pidempleado) empleado
			JOIN laboreo_pwa.festivo ON festivo.idempleado = empleado.idempleado
			WHERE extract(year from festivo.fechainicio) = extract(year from pfecha)
				AND (festivo.autorizada = TRUE OR (festivo.autorizada = FALSE AND festivo.fechaautorizacion IS NULL))
			GROUP BY festivo.idempleado
	) vacaciones
		ON vacaciones.idempleado = empleado.idempleado;
END;
$BODY$;

ALTER FUNCTION laboreo_pwa.f_empleado_resumenfestivo(bigint, timestamp without time zone)
    OWNER TO prbnnoeimelclv;

/* Resumen vacaciones */