DROP SCHEMA laboreo_pwa CASCADE;
BEGIN;
-- Creación de un schema
CREATE SCHEMA laboreo_pwa; --AUTHORIZATION userpwa

-- Asignar permisos 
GRANT USAGE ON SCHEMA laboreo_pwa TO userpwa;
GRANT ALL ON ALL TABLES IN SCHEMA laboreo_pwa TO userpwa;
GRANT USAGE, SELECT ON ALL SEQUENCES IN SCHEMA laboreo_pwa TO userpwa;
	
-- Tabla de Organización (Negocio)
CREATE TABLE IF NOT EXISTS laboreo_pwa.organizacion (
	idOrganizacion BIGSERIAL NOT NULL,
	nombre VARCHAR(100) NOT NULL,
	logotipo VARCHAR(150),
	cif VARCHAR(20) not null,
	registro VARCHAR(200),
	CONSTRAINT pk_organizacion_id PRIMARY KEY (idOrganizacion)
) WITHOUT OIDS;

-- Tabla de rol de usuario
CREATE TABLE IF NOT EXISTS laboreo_pwa.rolUsuario (
	idRolUsuario BIGSERIAL NOT NULL,
	codigo VARCHAR(5) NOT NULL,
	nombre VARCHAR(100) NOT NULL,
	administrador BOOLEAN NOT NULL DEFAULT FALSE,
	supervisor BOOLEAN NOT NULL DEFAULT FALSE,
	CONSTRAINT pk_rolUsuario_id PRIMARY KEY (idRolUsuario),
	CONSTRAINT uq_rolUsuario_codigo UNIQUE (codigo)
) WITHOUT OIDS;
--rollback;

-- Tabla para Centro de Trabajo
CREATE TABLE IF NOT EXISTS laboreo_pwa.centroTrabajo (
	idCentroTrabajo BIGSERIAL NOT NULL,
	idOrganizacion BIGINT NOT NULL,
	codigo VARCHAR(5) NOT NULL,
	nombre VARCHAR(100) NOT NULL,	
	direccion VARCHAR(150) NOT NULL,
	codigoPostal VARCHAR(10) NOT NULL,
	poblacion VARCHAR(100) NOT NULL,
	pais VARCHAR(50) NOT NULL,
	telefono VARCHAR(20) NOT NULL,
	CONSTRAINT pk_centroTrabajo_id PRIMARY KEY (idCentroTrabajo),
	CONSTRAINT uq_centroTrabajo_codigo UNIQUE (codigo),
	CONSTRAINT fk_centroTrabajo_idOrganizacion FOREIGN KEY (idOrganizacion) REFERENCES laboreo_pwa.organizacion(idOrganizacion)
) WITHOUT OIDS;

-- Tabla para Convenio
CREATE TABLE IF NOT EXISTS laboreo_pwa.convenio (
	idConvenio BIGSERIAL NOT NULL,
	idOrganizacion BIGINT NOT NULL,
	codigo VARCHAR(5) NOT NULL,
	nombre VARCHAR(100) NOT NULL,	
	descripcion VARCHAR(150) NOT NULL,
	diasNaturalesVacaciones INTEGER NOT NULL,
	diasLaborablesVacaciones INTEGER NOT NULL,
	minutosJornada INTEGER NOT NULL,	
	CONSTRAINT pk_convenio_id PRIMARY KEY (idConvenio),
	CONSTRAINT uq_convenio_codigo UNIQUE (codigo),
	CONSTRAINT fk_convenio_idOrganizacion FOREIGN KEY (idOrganizacion) REFERENCES laboreo_pwa.organizacion(idOrganizacion)
) WITHOUT OIDS;

-- Tabla para Tipo Contrato
CREATE TABLE IF NOT EXISTS laboreo_pwa.tipoContrato (
	idTipoContrato BIGSERIAL NOT NULL,
	idOrganizacion BIGINT NOT NULL,
	codigo VARCHAR(5) NOT NULL,
	nombre VARCHAR(100) NOT NULL,	
	CONSTRAINT pk_tipoContrato_id PRIMARY KEY (idTipoContrato),
	CONSTRAINT uq_tipoContrato_codigo UNIQUE (codigo),
	CONSTRAINT fk_tipoContrato_idOrganizacion FOREIGN KEY (idOrganizacion) REFERENCES laboreo_pwa.organizacion(idOrganizacion)
) WITHOUT OIDS;

-- Tabla para Turno
CREATE TABLE IF NOT EXISTS laboreo_pwa.turno (
	idTurno BIGSERIAL NOT NULL,
	idOrganizacion BIGINT NOT NULL,
	codigo VARCHAR(5) NOT NULL,
	nombre VARCHAR(100) NOT NULL,
	lunes BOOLEAN,
	martes BOOLEAN,
	miercoles BOOLEAN,
	jueves BOOLEAN,
	viernes BOOLEAN,
	sabado BOOLEAN,
	domingo BOOLEAN,
	horaInicio TIME,
	horaFin TIME,
	CONSTRAINT pk_turno_id PRIMARY KEY (idTurno),
	CONSTRAINT uq_turno_codigo UNIQUE (codigo),
	CONSTRAINT fk_turno_idOrganizacion FOREIGN KEY (idOrganizacion) REFERENCES laboreo_pwa.organizacion(idOrganizacion)
) WITHOUT OIDS;

-- Tabla para Departamento
CREATE TABLE IF NOT EXISTS laboreo_pwa.departamento (
	idDepartamento BIGSERIAL NOT NULL,
	idcentrotrabajo BIGINT NOT NULL,
	idEmpleado_responsable BIGINT,
	codigo VARCHAR(5) NOT NULL,
	nombre VARCHAR(100) NOT NULL,	
	CONSTRAINT pk_departamento_id PRIMARY KEY (idDepartamento),
	CONSTRAINT uq_departamento_codigo UNIQUE (codigo),
	CONSTRAINT fk_departamento_idcentrotrabajo FOREIGN KEY (idCentroTrabajo) REFERENCES laboreo_pwa.centroTrabajo(idCentroTrabajo)
) WITHOUT OIDS;

-- Tabla de empleados
CREATE TABLE IF NOT EXISTS laboreo_pwa.empleado (
	idEmpleado BIGSERIAL NOT NULL,
	idOrganizacion BIGINT NOT NULL,
	idDepartamento BIGINT NOT NULL,
	idConvenio BIGINT NOT NULL,	
	idContratoEmpleado_actual BIGINT,
	idRolUsuario BIGINT NOT NULL,
	nombre VARCHAR(100) NOT NULL,
	apellidos VARCHAR(100) NOT NULL,
	dni VARCHAR(20) NOT NULL,
	telefono VARCHAR(20),
	email VARCHAR(80),
	observaciones TEXT,
	pin VARCHAR(4) NOT NULL,
	foto VARCHAR(150),
	numeroINSS VARCHAR(20),
	direccion VARCHAR(150) NOT NULL,
	codigoPostal VARCHAR(10) NOT NULL,
	poblacion VARCHAR(100) NOT NULL,
	pais VARCHAR(50) NOT NULL,
	CONSTRAINT pk_empleado_id PRIMARY KEY (idEmpleado),
	CONSTRAINT uq_empleado_dni UNIQUE (dni),
	CONSTRAINT fk_empleado_idOrganizacion FOREIGN KEY (idOrganizacion) REFERENCES laboreo_pwa.organizacion(idOrganizacion),	
	CONSTRAINT fk_empleado_idDepartamento FOREIGN KEY (idDepartamento) REFERENCES laboreo_pwa.departamento(idDepartamento),
	CONSTRAINT fk_empleado_idConvenio FOREIGN KEY (idConvenio) REFERENCES laboreo_pwa.convenio(idConvenio),
	CONSTRAINT fk_empleado_idRolUsuario FOREIGN KEY (idRolUsuario) REFERENCES laboreo_pwa.rolUsuario(idRolUsuario)
) WITHOUT OIDS;


-- Tabla de Contrato de empleado
CREATE TABLE IF NOT EXISTS laboreo_pwa.contratoEmpleado (
	idContratoEmpleado BIGSERIAL NOT NULL,
	idTipoContrato BIGINT NOT NULL,
	idEmpleado BIGINT NOT NULL,
	fechaInicio DATE NOT NULL,
	fechaFin DATE NOT NULL,	
	CONSTRAINT pk_contratoEmpleado_id PRIMARY KEY (idContratoEmpleado),
	CONSTRAINT fk_contratoEmpleado_idTipoContrato FOREIGN KEY (idTipoContrato) REFERENCES laboreo_pwa.tipoContrato(idTipoContrato),
	CONSTRAINT fk_contratoEmpleado_idEmpleado FOREIGN KEY (idEmpleado) REFERENCES laboreo_pwa.empleado(idEmpleado)
) WITHOUT OIDS;

-- Tabla de jornadas
CREATE TABLE IF NOT EXISTS laboreo_pwa.jornada (
	idJornada BIGSERIAL NOT NULL,
	idTurno BIGINT NOT NULL,
	idEmpleado BIGINT NOT NULL,
	fechaInicio DATE NOT NULL,
	fechaFin DATE NOT NULL,	
	CONSTRAINT pk_jornada_id PRIMARY KEY (idJornada),
	CONSTRAINT fk_jornada_idTurno FOREIGN KEY (idTurno) REFERENCES laboreo_pwa.turno(idTurno),
	CONSTRAINT fk_jornada_idEmpleado FOREIGN KEY (idEmpleado) REFERENCES laboreo_pwa.empleado(idEmpleado)
) WITHOUT OIDS;

-- Tabla de jornadas
CREATE TABLE IF NOT EXISTS laboreo_pwa.evento (
	idEvento BIGSERIAL NOT NULL,
	idEmpleado BIGINT NOT NULL,
	fechaInicio TYPE TIMESTAMP WITHOUT TIME ZONE NOT NULL,
	fechaFin TYPE TIMESTAMP WITHOUT TIME ZONE NOT NULL,	
	CONSTRAINT pk_evento_id PRIMARY KEY (idEvento),
	CONSTRAINT fk_evento_idEmpleado FOREIGN KEY (idEmpleado) REFERENCES laboreo_pwa.empleado(idEmpleado)
) WITHOUT OIDS;
-- Tabla para vacaciones
CREATE TABLE IF NOT EXISTS laboreo_pwa.festivo  ( 
	idEmpleado_autoriza BIGINT,
	autorizada BOOLEAN NULL, 
	fechaAutorizacion DATE NULL,
	motivoRechazo VARCHAR(300) NULL,
	CONSTRAINT fk_festivo_idEmpleado_autoriza FOREIGN KEY (idEmpleado_autoriza) REFERENCES laboreo_pwa.empleado(idEmpleado)
)
INHERITS (laboreo_pwa.evento);
-- Tabla para registros horarios
CREATE TABLE IF NOT EXISTS laboreo_pwa.registroHorario  ( 
	entrada BOOLEAN NOT NULL, 
	longitud FLOAT,
	latitud FLOAT,
	fechaRegistro TYPE TIMESTAMP WITHOUT TIME ZONE NOT NULL DEFAULT NOW()
  ) 
INHERITS (laboreo_pwa.evento);
-- Tabla para ausencia
CREATE TABLE IF NOT EXISTS laboreo_pwa.ausencia  ( 
	motivo VARCHAR(200),
	minutos INTEGER
  ) 
INHERITS (laboreo_pwa.evento);
-- Tabla para personal
CREATE TABLE IF NOT EXISTS laboreo_pwa.personal  ( 
	titulo VARCHAR(50),
	descripcion VARCHAR(200)	
  ) 
INHERITS (laboreo_pwa.evento);

-- Tabla de aviso
CREATE TABLE IF NOT EXISTS laboreo_pwa.Aviso (
	idAviso BIGSERIAL NOT NULL,
	idEmpleado BIGINT NOT NULL,
	titulo VARCHAR(50),
	descripcion VARCHAR(200),
	leido BOOLEAN NOT NULL DEFAULT FALSE,
	CONSTRAINT pk_aviso_id PRIMARY KEY (idAviso),	
	CONSTRAINT fk_aviso_idEmpleado FOREIGN KEY (idEmpleado) REFERENCES laboreo_pwa.empleado(idEmpleado)
) WITHOUT OIDS;

-- Vista para el calendario
CREATE OR REPLACE VIEW laboreo_pwa.calendarioEmpleadoV AS 
SELECT evento.idEmpleado AS idEmpleado
	, evento.fechaInicio AS fechaInicio
	, evento.fechaFin AS fechaFin
	, 'Vacaciones' AS eventoNombre
	, 'festivo' AS eventoTipo
FROM laboreo_pwa.evento
INNER JOIN laboreo_pwa.festivo
	ON evento.idEvento = festivo.idEvento
UNION
SELECT evento.idEmpleado AS idEmpleado
	, evento.fechaInicio AS fechaInicio
	, evento.fechaFin AS fechaFin
	, CASE WHEN registroHorario.entrada THEN 'Entrada' ELSE 'Salida' END AS eventoNombre
	, 'registroHorario' AS eventoTipo
FROM laboreo_pwa.evento
INNER JOIN laboreo_pwa.registroHorario
	ON evento.idEvento = registroHorario.idEvento
UNION
SELECT evento.idEmpleado AS idEmpleado
	, evento.fechaInicio AS fechaInicio
	, evento.fechaFin AS fechaFin
	, ausencia.motivo AS eventoNombre
	, 'ausencia' AS eventoTipo
FROM laboreo_pwa.evento
INNER JOIN laboreo_pwa.ausencia
	ON evento.idEvento = ausencia.idEvento
UNION
SELECT evento.idEmpleado AS idEmpleado
	, evento.fechaInicio AS fechaInicio
	, evento.fechaFin AS fechaFin
	, personal.titulo AS eventoNombre
	, 'personal' AS eventoTipo
FROM laboreo_pwa.evento
INNER JOIN laboreo_pwa.personal
	ON evento.idEvento = personal.idEvento;

-- Creamos foránea en departamento para empleado responsable
ALTER TABLE laboreo_pwa.departamento 
ADD CONSTRAINT fk_departamento_idEmpleado_responsable 
FOREIGN KEY (idEmpleado_responsable) 
REFERENCES laboreo_pwa.empleado(idEmpleado);

-- Creamos foránea en empleado para último contrato
ALTER TABLE laboreo_pwa.empleado 
ADD CONSTRAINT fk_empleado_idContratoEmpleado_actual
FOREIGN KEY (idContratoEmpleado_actual) 
REFERENCES laboreo_pwa.contratoEmpleado(idContratoEmpleado);

-- Insertar valores por defecto
INSERT INTO laboreo_pwa.rolUsuario (codigo, nombre, administrador, supervisor)
SELECT 'ADMIN', 'Administrador', TRUE, TRUE
WHERE NOT EXISTS (SELECT * FROM laboreo_pwa.rolUsuario WHERE codigo = 'ADMIN');

INSERT INTO laboreo_pwa.rolUsuario (codigo, nombre, administrador, supervisor)
SELECT 'SUPER', 'Supervisor', FALSE, TRUE
WHERE NOT EXISTS (SELECT * FROM laboreo_pwa.rolUsuario WHERE codigo = 'SUPER');

INSERT INTO laboreo_pwa.rolUsuario (codigo, nombre, administrador, supervisor)
SELECT 'USER', 'Usuario', FALSE, FALSE
WHERE NOT EXISTS (SELECT * FROM laboreo_pwa.rolUsuario WHERE codigo = 'USER');



-- Vistas para permisos --
-- centrotrabajov
CREATE OR REPLACE VIEW laboreo_pwa.centrotrabajov AS 
SELECT centrotrabajo.*, empleado.idempleado AS idempleadoacceso
FROM laboreo_pwa.empleado
INNER JOIN laboreo_pwa.rolusuario
	ON rolusuario.idrolusuario = empleado.idrolusuario
INNER JOIN laboreo_pwa.centrotrabajo
	ON centrotrabajo.idorganizacion = empleado.idorganizacion
INNER JOIN laboreo_pwa.departamento
	ON departamento.idcentrotrabajo = centrotrabajo.idcentrotrabajo
WHERE  rolusuario.administrador = true
	OR (rolusuario.supervisor = true AND departamento.idempleado_responsable = empleado.idempleado);


-- departamentov
CREATE OR REPLACE VIEW laboreo_pwa.departamentov AS 
SELECT departamento.*, empleado.idempleado AS idempleadoacceso
FROM laboreo_pwa.empleado
INNER JOIN laboreo_pwa.rolusuario
	ON rolusuario.idrolusuario = empleado.idrolusuario
INNER JOIN laboreo_pwa.centrotrabajo
	ON centrotrabajo.idorganizacion = empleado.idorganizacion
INNER JOIN laboreo_pwa.departamento
	ON departamento.idcentrotrabajo = centrotrabajo.idcentrotrabajo
WHERE rolusuario.administrador = true
	OR (rolusuario.supervisor = true AND departamento.idempleado_responsable = empleado.idempleado);	


-- empleadov
-- View: laboreo_pwa.empleadov
-- DROP VIEW laboreo_pwa.empleadov;
CREATE OR REPLACE VIEW laboreo_pwa.empleadov
 AS
 SELECT empleado.idempleado,
    empleado.idorganizacion,
    empleado.iddepartamento,
    empleado.idconvenio,
    empleado.idcontratoempleado_actual,
    empleado.idrolusuario,
    empleado.nombre,
    empleado.dni,
    empleado.telefono,
    empleado.email,
    empleado.observaciones,
    empleado.pin,
    empleado.foto,
    empleado.numeroinss,
    empleado.direccion,
    empleado.codigopostal,
    empleado.poblacion,
    empleado.pais,
    empleado.apellidos,
    COALESCE(empleado.foto, 'img/avatar-noimage.png'::character varying) AS imagen,
    departamento.idcentrotrabajo,
    empleadosupervisor.idempleado AS idempleadoacceso,
	avisos.pendientes
   FROM laboreo_pwa.empleado
     JOIN laboreo_pwa.departamento ON departamento.iddepartamento = empleado.iddepartamento
     JOIN laboreo_pwa.empleado empleadosupervisor ON empleadosupervisor.idempleado = departamento.idempleado_responsable
	 LEFT JOIN (SELECT COUNT(*) AS pendientes, aviso.idempleado
				FROM laboreo_pwa.aviso
				WHERE aviso.leido = false 
				GROUP BY aviso.idempleado) avisos 
		ON avisos.idempleado = empleado.idempleado;

ALTER TABLE laboreo_pwa.empleadov
    OWNER TO prbnnoeimelclv;
-- empleadov



/***** TRIGGERS ******/

/** avisos de vacaciones **/
CREATE OR REPLACE  FUNCTION laboreo_pwa.aviso_festivo() RETURNS trigger AS $aviso_festivo$
    BEGIN
		-- Insertamos un nuevo aviso
		INSERT INTO laboreo_pwa.aviso (idempleado, titulo, descripcion, leido)
		SELECT 	empleadov.idempleadoacceso, 
				'Solicitud de Vacaciones',
				'Solicitud de vacaciones pendiente de verificar del empleado ' || empleadov.nombre || ' (' || to_char(NEW.fechainicio, 'DD/MM/YYYY')::varchar || ' - ' || to_char(NEW.fechafin, 'DD/MM/YYYY')::varchar ||' )',
				false
		FROM laboreo_pwa.empleadov
		WHERE empleadov.idempleado = NEW.idempleado;
		
        RETURN NEW;
    END;
$aviso_festivo$ LANGUAGE plpgsql;

CREATE TRIGGER aviso_festivo AFTER INSERT ON laboreo_pwa.festivo
    FOR EACH ROW EXECUTE PROCEDURE laboreo_pwa.aviso_festivo();
/** avisos de vacaciones **/	

/** avisos de confirmación de vacaciones **/
CREATE FUNCTION laboreo_pwa.aviso_confirmacionfestivo()
    RETURNS trigger
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE NOT LEAKPROOF
AS $BODY$
    BEGIN
		-- Insertamos un nuevo aviso
		INSERT INTO laboreo_pwa.aviso (idempleado, titulo, descripcion, leido)
		SELECT 	NEW.idempleado, 
				'Confirmación de Vacaciones',
				'La solicitud de vacaciones pendiente de verificar se ha ' || CASE WHEN NEW.autorizada = TRUE THEN 'AUTORIZADO' ELSE 'RECHAZADO' END || ' (' || to_char(NEW.fechainicio, 'DD/MM/YYYY')::varchar || ' - ' || to_char(NEW.fechafin, 'DD/MM/YYYY')::varchar ||' )',
				false
		FROM laboreo_pwa.empleadov
		WHERE empleadov.idempleado = NEW.idempleado
			AND ((NEW.fechaautorizacion <> OLD.fechaautorizacion) OR OLD.fechaautorizacion IS NULL) ;
		
        RETURN NEW;
    END;
$BODY$;

ALTER FUNCTION laboreo_pwa.aviso_confirmacionfestivo()
    OWNER TO prbnnoeimelclv;
/** avisos de vacaciones **/



commit;