const express = require('express');
const router = express.Router();  // Objeto para definir rutas

// Conexión con PostgreSQL
const pool = require('./database/pool');

// GET all
router.get('/', (request, response) => {
    pool.query('SELECT * FROM laboreo_pwa.aviso ORDER BY leido ASC', (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});
  
// GET id
router.get('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('SELECT * FROM laboreo_pwa.aviso WHERE idaviso = $1', [id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});


// PUT modificación
router.put('/:id', (request, response) => {
    const {id} = request.params;
    const { titulo, descripcion, leido, idempleado } = request.body;
    if (leido) {
        pool.query('UPDATE laboreo_pwa.aviso SET titulo = $1, descripcion = $2, leido = $3, $idempleado = $4 WHERE idaviso = $5;', [titulo, descripcion, leido, idempleado, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${id}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});

// PATCH confirmar
router.patch('/:id/confirmar', (request, response) => {
    const {id} = request.params;            
    pool.query('UPDATE laboreo_pwa.aviso SET leido = true WHERE idaviso = $1;', [id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${id}`)
    });             

});

// POST insertar
router.post('/', (request, response) => {
    const { idempleado, titulo, descripcion, leido } = request.body;
//    console.log(request.body);    
//    console.log(`INSERT INTO laboreo_pwa.aviso (idempleado, fechainicio, fechafin, entrada, longitud, latitud, fecharegistro) VALUES (${idempleado}, ${fechainicio}, ${fechafin}, ${entrada}, ${longitud}, ${latitud});`);
    if (idempleado && titulo && descripcion) {
        pool.query('INSERT INTO laboreo_pwa.aviso (idempleado, titulo, descripcion, leido) VALUES ($1, $2, $3, false);', [idempleado, titulo, descripcion], (error, results) => {
        if (error) {         
            response.status(500).send(error);
            //throw error;
        }
        //response.status(201).send(`User added with ID: ${results.insertId}`);
        response.status(201).json({statustext: 'Registro insertado correctamente'});
        });           

    } else {     
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});


router.delete('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('DELETE FROM laboreo_pwa.aviso WHERE idaviso = $1', [id], (error, results) => {
        if (error) {
            response.status(500).send(error);      
        }
        response.status(200).send(`Registro horario eliminado con ID: ${id}`)
    })
});

module.exports = router;