const express = require('express');
const router = express.Router();  // Objeto para definir rutas

// Conexión con PostgreSQL
const pool = require('./database/pool');

// GET all
router.get('/', (request, response) => {
    console.log(`SELECT * FROM laboreo_pwa.f_empleado(${request.user});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado($1) ORDER BY idempleado ASC', [request.user], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});
  
// GET id
router.get('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('SELECT * FROM laboreo_pwa.empleadov WHERE idempleado = $1 ORDER BY idempleado ASC', [id], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});

// GET id - Registros horarios de un empleado
router.get('/:id/registroshorarios', (request, response) => {
    const {id} = request.params;
    console.log(id)
    console.log(`SELECT * FROM laboreo_pwa.registrohorario WHERE idempleado =  ${id};`);
    pool.query('SELECT * FROM laboreo_pwa.registrohorario WHERE idempleado = $1 ORDER BY fechainicio DESC', [id], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});

// GET id - Registros horarios por fechas
router.get('/:id/registroshorarios/:fechainicio/:fechafin', (request, response) => {
    const {id, fechainicio, fechafin} = request.params;
    console.log(id, fechainicio, fechafin)
    console.log(`SELECT * FROM laboreo_pwa.registrohorario WHERE idempleado =  ${id} AND fechainicio::date >= ${fechainicio} AND fechafin::date <= ${fechafin};`);
    pool.query('SELECT * FROM laboreo_pwa.registrohorario WHERE idempleado = $1 AND fechainicio::date >= $2 AND fechafin::date <= $3 ORDER BY fechainicio DESC', [id, fechainicio, fechafin], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});

// GET id - Calendario empleado por fechas
router.get('/:id/calendarioempleado/:fechainicio/:fechafin', (request, response) => {
    const {id, fechainicio, fechafin} = request.params;
    console.log(id, fechainicio, fechafin)
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_calendario(${id}, ${fechainicio}, ${fechafin});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_calendario($1, $2, $3) ORDER BY dia', [id, fechainicio, fechafin], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});


// GET id - Eventos empleado por fechas
router.get('/:id/calendarioevento/:fechainicio/:fechafin', (request, response) => {
    const {id, fechainicio, fechafin} = request.params;
    console.log(id, fechainicio, fechafin)
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_calendarioevento(${id}, ${fechainicio}, ${fechafin});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_calendarioevento($1, $2, $3) ORDER BY idempleado, fechainicio', [id, fechainicio, fechafin], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});


// GET id - Contratos del empleado
router.get('/:id/contratosempleado', (request, response) => {
    const {id} = request.params;
    console.log(id)
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_contratos(${id});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_contratos($1) ORDER BY fechainicio DESC', [id], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});


// GET id - Jornadas del empleado
router.get('/:id/jornadas', (request, response) => {
    const {id} = request.params;
    console.log(id)
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_jornadas(${id});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_jornadas($1) ORDER BY fechainicio DESC', [id], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});


// GET id - Jornadas del empleado
router.get('/:id/turnos', (request, response) => {
    const {id} = request.params;
    console.log(id)
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_turnos(${id});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_turnos($1) ORDER BY fechainicio ASC', [id], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});


// GET id - Informe mensual
router.get('/:id/informemensual/:fechainicio/:fechafin', (request, response) => {
    const { id, fechainicio, fechafin } = request.params;        
    //console.log(id, mes, anyo);
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_resumenmes(${id}, ${fechainicio}, ${fechafin});`);
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_resumenmes($1, $2, $3) ORDER BY dia', [id, fechainicio, fechafin], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});

// GET id - Informe ausencias
router.get('/:id/informeausencia/:fechainicio/:fechafin', (request, response) => {
    const { id, fechainicio, fechafin } = request.params;        
    //console.log(id, mes, anyo);
    console.log(`SELECT * FROM laboreo_pwa.f_empleado_resumenmes(${id}, ${fechainicio}, ${fechafin});`);
    pool.query('SELECT * FROM laboreo_pwa.ausencia WHERE idempleado = $1 AND fechainicio::date >= $2 AND fechafin::date <= $3 ORDER BY fechainicio DESC', [id, fechainicio, fechafin], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});

// GET id - Informe festivo
router.get('/:id/informefestivo/:fecha', (request, response) => {
    const { id, fecha } = request.params;        
    //console.log(id, mes, anyo);    
    pool.query('SELECT * FROM laboreo_pwa.f_empleado_resumenfestivo($1, $2) ORDER BY apellidos, nombre DESC', [id, fecha], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });

});


// GET id - Avisos
router.get('/:id/avisos', (request, response) => {
    const {id} = request.params;
    console.log(id)
    console.log(`SELECT * FROM laboreo_pwa.aviso WHERE idempleado =  ${id};`);
    pool.query('SELECT * FROM laboreo_pwa.aviso WHERE idempleado = $1 AND leido = false', [id], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});

// GET id - Avisos
router.get('/:id/status', (request, response) => {
    const {id} = request.params;
    console.log(id)
    console.log(`SELECT COUNT(*) FROM laboreo_pwa.aviso WHERE idempleado =  ${id};`);
    pool.query('SELECT CASE WHEN COUNT(*)>0 THEN COUNT(*) ELSE NULL END AS pendientes FROM laboreo_pwa.aviso WHERE idempleado = $1 AND leido = false', [id], (error, results) => {
            if (error) {
            //throw error
            response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});


// PUT modificación
router.put('/:id', (request, response) => {
    const {id} = request.params;
    const { nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idcontratoempleado_actual, idrolusuario } = request.body;
    //const { imagen } = request.file.path;
    //console.log(`UPDATE laboreo_pwa.empleado SET nombre = ${nombre}, apellidos = ${apellidos}, dni  = ${dni}, telefono = ${telefono}, email = ${email}, observaciones = ${observaciones},  pin = ${pin}, foto = ${foto}, numeroinss = ${numeroinss}, direccion = ${direccion}, codigopostal =  ${codigopostal}, poblacion = ${poblacion}, pais = ${pais}, idorganizacion = ${idorganizacion}, iddepartamento = ${iddepartamento}, idconvenio = ${idconvenio}, idrolusuario = ${idrolusuario} WHERE idempleado = $${id};`);
    if (nombre && apellidos && dni && email && pin && direccion && codigopostal && poblacion && pais && idorganizacion && iddepartamento && idconvenio && idrolusuario) {
        pool.query('UPDATE laboreo_pwa.empleado SET nombre = $1, apellidos = $2, dni  = $3, telefono = $4, email = $5, observaciones = $6,  pin = $7, foto = $8, numeroinss = $9, direccion = $10, codigopostal = $11, poblacion = $12, pais = $13, idorganizacion = $14, iddepartamento = $15, idconvenio = $16, idrolusuario = $17 WHERE idempleado = $18;', [nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idrolusuario, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json({statustext: 'Registro modificado correctamente'})
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});
// PATCH
router.patch('/:id', (request, response) => {
    const {id} = request.params;
    const { nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idcontratoempleado_actual, idrolusuario } = request.body;
    if (nombre && apellidos && dni && telefono && email && pin && direccion && codigopostal && poblacion && pais && idorganizacion && iddepartamento && idconvenio && idrolusuario) {
        pool.query('UPDATE laboreo_pwa.empleado SET nombre = $1, apellidos = $2, dni  = $3, telefono = $4, email = $5, observaciones = $6,  pin = $7, foto = $8, numeroinss = $9, direccion = $10, codigopostal = $11, poblacion = $12, pais = $13, idorganizacion = $14, iddepartamento = $15, idconvenio = $16, idrolusuario = $17 WHERE idempleado = $18;', [nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idrolusuario, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`User modified with ID: ${id}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});

// POST insertar
router.post('/', (request, response) => {
    const { nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idcontratoempleado_actual, idrolusuario } = request.body;
    //console.log(`INSERT INTO laboreo_pwa.empleado (nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idcontratoempleado_actual, idrolusuario) VALUES (${nombre}, ${apellidos}, ${dni}, ${telefono}, ${email}, ${observaciones}, ${pin}, ${foto}, ${numeroinss}, ${direccion}, ${codigopostal}, ${poblacion}, ${pais}, ${idorganizacion}, ${iddepartamento}, ${idconvenio}, ${idcontratoempleado_actual}, ${idrolusuario});`);
    if (nombre && apellidos && dni && email && pin && idorganizacion && iddepartamento && idconvenio && idrolusuario && direccion && codigopostal && poblacion && pais) {
        pool.query('INSERT INTO laboreo_pwa.empleado (nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idcontratoempleado_actual, idrolusuario) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17, $18);', [nombre, apellidos, dni, telefono, email, observaciones, pin, foto, numeroinss, direccion, codigopostal, poblacion, pais, idorganizacion, iddepartamento, idconvenio, idcontratoempleado_actual, idrolusuario], (error, results) => {
        if (error) {         
            response.status(500).send(error);
            //throw error;
        }
        response.status(201).send(`User added with ID: ${results.insertId}`);
        });           

    } else {     
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});


router.delete('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('DELETE FROM laboreo_pwa.empleado WHERE idempleado = $1', [id], (error, results) => {
        if (error) {
        response.status(500).send(error);      
        }
        response.status(200).send(`User deleted with ID: ${id}`)
    })
});

module.exports = router;