const express = require('express');
const router = express.Router();  // Objeto para definir rutas

// Conexión con PostgreSQL
const pool = require('./database/pool');

// GET all
router.get('/', (request, response) => {
    pool.query('SELECT * FROM laboreo_pwa.jornada ORDER BY codigo ASC', (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});
  
// GET id
router.get('/:id', (request, response) => {
    const {id} = request.params;
    pool.query("SELECT jornada.idjornada, to_char(jornada.fechainicio, 'YYYY-MM-DD')::varchar AS fechainicio, to_char(jornada.fechafin, 'YYYY-MM-DD')::varchar AS fechafin, jornada.idempleado, jornada.idturno FROM laboreo_pwa.jornada WHERE idjornada = $1", [id], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});


// PUT modificación
router.put('/:id', (request, response) => {
    const {id} = request.params;
    const { fechainicio, fechafin, idempleado, idturno } = request.body;
    if (fechainicio && fechafin && idempleado && idturno) {
        pool.query('UPDATE laboreo_pwa.jornada SET fechainicio = $1, fechafin = $2, idempleado  = $3, idturno = $4 WHERE idjornada = $5;', [fechainicio, fechafin, idempleado, idturno, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${id}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});
// PATCH
router.patch('/:id', (request, response) => {
    const {id} = request.params;
    const { fechainicio, fechafin, idempleado, idturno } = request.body;
    if (fechainicio && fechafin && idempleado && idturno) {
        pool.query('UPDATE laboreo_pwa.jornada SET fechainicio = $1, fechafin = $2, idempleado  = $3, idturno = $4 WHERE idjornada = $5;', [fechainicio, fechafin, idempleado, idturno, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${idevento}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});

// POST insertar
router.post('/', (request, response) => {
    const { fechainicio, fechafin, idempleado, idturno } = request.body;
//    console.log(request.body);    
//    console.log(`INSERT INTO laboreo_pwa.jornada (idempleado, fechainicio, fechafin, entrada, longitud, latitud, fecharegistro) VALUES (${idempleado}, ${fechainicio}, ${fechafin}, ${entrada}, ${longitud}, ${latitud});`);
    if (idempleado && fechainicio && fechafin && idturno) {
        pool.query('INSERT INTO laboreo_pwa.jornada (fechainicio, fechafin, idempleado, idturno) VALUES ($1, $2, $3, $4);', [fechainicio, fechafin, idempleado, idturno], (error, results) => {
        if (error) {         
            response.status(500).send(error);
            //throw error;
        }
        //response.status(201).send(`User added with ID: ${results.insertId}`);
        response.status(201).json({status: 'Registrado'});
        });           

    } else {     
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});


router.delete('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('DELETE FROM laboreo_pwa.jornada WHERE idjornada = $1', [id], (error, results) => {
        if (error) {
            response.status(500).send(error);      
        }
        response.status(200).send(`Registro horario eliminado con ID: ${id}`)
    })
});

module.exports = router;