const express = require('express');
const router = express.Router();  // Objeto para definir rutas

// Conexión con PostgreSQL
const pool = require('./database/pool');

// GET all
router.get('/', (request, response) => {
    pool.query('SELECT * FROM laboreo_pwa.organizacion', (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});
  
// GET id
router.get('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('SELECT * FROM laboreo_pwa.organizacion WHERE idorganizacion = $1', [id], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});

// GET id
router.get('/:id/departamentos', (request, response) => {    
    const {id} = request.params;
    pool.query('SELECT * FROM laboreo_pwa.departamento WHERE idorganizacion = $1', [id], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});

// GET id
router.get('/:id/empleados', (request, response) => {    
    const {id} = request.params;
    pool.query('SELECT empleado.*, COALESCE(empleado.foto, \'img/avatar-noimage.png\') AS imagen FROM laboreo_pwa.empleado INNER JOIN laboreo_pwa.departamento ON departamento.iddepartamento = empleado.iddepartamento WHERE departamento.idorganizacion = $1', [id], (error, results) => {
            if (error) {
                //throw error
                response.status(500).send(error);
            }
            response.status(200).json(results.rows)
    });
});


// PUT modificación
router.put('/:id', (request, response) => {
    const {id} = request.params;
    const { nombre, cif, registro, logotipo } = request.body;
    if (nombre && cif ) {
        pool.query('UPDATE laboreo_pwa.organizacion SET nombre = $1, cif = $2, registro  = $3, logotipo = $4 WHERE idorganizacion = $5;', [nombre, cif, registro, logotipo, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${id}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});
// PATCH
router.patch('/:id', (request, response) => {
    const {id} = request.params;
    const { nombre, cif, registro, logotipo } = request.body;
    if (nombre && cif ) {
        pool.query('UPDATE laboreo_pwa.organizacion SET nombre = $1, cif = $2, registro  = $3, logotipo = $4 WHERE idorganizacion = $5;', [nombre, cif, registro, logotipo, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${idevento}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});

// POST insertar
router.post('/', (request, response) => {
    const { nombre, cif, registro, logotipo } = request.body;
//    console.log(request.body);    
//    console.log(`INSERT INTO laboreo_pwa.organizacion (idempleado, fechainicio, fechafin, entrada, longitud, latitud, fecharegistro) VALUES (${idempleado}, ${fechainicio}, ${fechafin}, ${entrada}, ${longitud}, ${latitud});`);
    if (nombre && cif) {
        pool.query('INSERT INTO laboreo_pwa.organizacion (nombre, cif, registro, logotipo) VALUES ($1, $2, $3, $4);', [nombre, cif, registro, logotipo], (error, results) => {
        if (error) {         
            response.status(500).send(error);
            //throw error;
        }
        //response.status(201).send(`User added with ID: ${results.insertId}`);
        response.status(201).json({status: 'Registrado'});
        });           

    } else {     
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});


router.delete('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('DELETE FROM laboreo_pwa.organizacion WHERE idorganizacion = $1', [id], (error, results) => {
        if (error) {
            response.status(500).send(error);      
        }
        response.status(200).send(`Registro horario eliminado con ID: ${id}`)
    })
});

module.exports = router;