const express = require('express');
const router = express.Router();  // Objeto para definir rutas

// Conexión con PostgreSQL
const pool = require('./database/pool');

// GET all
router.get('/', (request, response) => {
    pool.query('SELECT * FROM laboreo_pwa.personal ORDER BY fechainicio ASC', (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});
  
// GET id
router.get('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('SELECT * FROM laboreo_pwa.personal WHERE idevento = $1', [id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).json(results.rows)
    });
});


// PUT modificación
router.put('/:id', (request, response) => {
    const {id} = request.params;
    const { fechainicio, fechafin, titulo, descripcion } = request.body;
    if (fechainicio && fechafin && titulo ) {
        pool.query('UPDATE laboreo_pwa.personal SET fechainicio = $1, fechafin = $2, titulo = $3, descripcion = $4 WHERE idevento = $5;', [fechainicio, fechafin, titulo, descripcion, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${id}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});
// PATCH
router.patch('/:id', (request, response) => {
    const {id} = request.params;
    const { fechainicio, fechafin, titulo, descripcion } = request.body;
    if (fechainicio && fechafin && titulo ) {
        pool.query('UPDATE laboreo_pwa.personal SET fechainicio = $1, fechafin = $2, titulo = $3, descripcion = $4 WHERE idevento = $5;', [fechainicio, fechafin, titulo, descripcion, id], (error, results) => {
        if (error) {
            //throw error
            response.status(500).send(error);
        }
        response.status(200).send(`Registro horario modificado con el ID: ${idevento}`)
        });             
    } else {
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});

// POST insertar
router.post('/', (request, response) => {
    const { idempleado, fechainicio, fechafin, titulo, descripcion } = request.body;
//    console.log(request.body);    
//    console.log(`INSERT INTO laboreo_pwa.personal (idempleado, fechainicio, fechafin, entrada, longitud, latitud, fecharegistro) VALUES (${idempleado}, ${fechainicio}, ${fechafin}, ${entrada}, ${longitud}, ${latitud});`);
    if (idempleado && fechainicio && titulo && descripcion) {
        pool.query('INSERT INTO laboreo_pwa.personal (idempleado, fechainicio, fechafin, titulo, descripcion) VALUES ($1, $2, $3, $4, $5);', [idempleado, fechainicio, fechafin, titulo, descripcion], (error, results) => {
        if (error) {         
            response.status(500).send(error);
            //throw error;
        }
        //response.status(201).send(`User added with ID: ${results.insertId}`);
        response.status(201).json({statustext: 'Registro insertado correctamente'});
        });           

    } else {     
        response.status(500).json({error: 'Ha ocurrido un error'});
    }
});


router.delete('/:id', (request, response) => {
    const {id} = request.params;
    pool.query('DELETE FROM laboreo_pwa.personal WHERE idevento = $1', [id], (error, results) => {
        if (error) {
            response.status(500).send(error);      
        }
        response.status(200).send(`Registro horario eliminado con ID: ${id}`)
    })
});

module.exports = router;